@@ -10,19 +10,22 @@ import sys
 import threading
 from collections import deque
 import collections.abc
-from contextlib import contextmanager, closing
+from contextlib import contextmanager
 import warnings
+import weakref
 import enum
 
 from contextvars import copy_context
 from math import inf
 from time import perf_counter
+from typing import Callable, TYPE_CHECKING
 
 from sniffio import current_async_library_cvar
 
 import attr
+from heapq import heapify, heappop, heappush
 from sortedcontainers import SortedDict
-from outcome import Error, Value, capture
+from outcome import Error, Outcome, Value, capture
 
 from ._entry_queue import EntryQueue, TrioToken
 from ._exceptions import TrioInternalError, RunFinishedError, Cancelled
@@ -35,15 +38,20 @@ from ._multierror import MultiError
 from ._traps import (
     Abort,
     wait_task_rescheduled,
+    cancel_shielded_checkpoint,
     CancelShieldedCheckpoint,
     PermanentlyDetachCoroutineObject,
     WaitTaskRescheduled,
 )
+from ._asyncgens import AsyncGenerators
 from ._thread_cache import start_thread_soon
+from ._instrumentation import Instruments
 from .. import _core
-from .._deprecate import deprecated, warn_deprecated
+from .._deprecate import warn_deprecated
 from .._util import Final, NoPublicConstructor, coroutine_or_error
 
+DEADLINE_HEAP_MIN_PRUNE_THRESHOLD = 1000
+
 _NO_SEND = object()
 
 
@@ -61,9 +69,6 @@ def _public(fn):
 _ALLOW_DETERMINISTIC_SCHEDULING = False
 _r = random.Random()
 
-# Used to log exceptions in instruments
-INSTRUMENT_LOGGER = logging.getLogger("trio.abc.Instrument")
-
 
 # On 3.7+, Context.run() is implemented in C and doesn't show up in
 # tracebacks. On 3.6, we use the contextvars backport, which is
@@ -123,6 +128,75 @@ class IdlePrimedTypes(enum.Enum):
 ################################################################
 
 
+@attr.s(eq=False, slots=True)
+class Deadlines:
+    """A container of deadlined cancel scopes.
+
+    Only contains scopes with non-infinite deadlines that are currently
+    attached to at least one task.
+
+    """
+
+    # Heap of (deadline, id(CancelScope), CancelScope)
+    _heap = attr.ib(factory=list)
+    # Count of active deadlines (those that haven't been changed)
+    _active = attr.ib(default=0)
+
+    def add(self, deadline, cancel_scope):
+        heappush(self._heap, (deadline, id(cancel_scope), cancel_scope))
+        self._active += 1
+
+    def remove(self, deadline, cancel_scope):
+        self._active -= 1
+
+    def next_deadline(self):
+        while self._heap:
+            deadline, _, cancel_scope = self._heap[0]
+            if deadline == cancel_scope._registered_deadline:
+                return deadline
+            else:
+                # This entry is stale; discard it and try again
+                heappop(self._heap)
+        return inf
+
+    def _prune(self):
+        # In principle, it's possible for a cancel scope to toggle back and
+        # forth repeatedly between the same two deadlines, and end up with
+        # lots of stale entries that *look* like they're still active, because
+        # their deadline is correct, but in fact are redundant. So when
+        # pruning we have to eliminate entries with the wrong deadline, *and*
+        # eliminate duplicates.
+        seen = set()
+        pruned_heap = []
+        for deadline, tiebreaker, cancel_scope in self._heap:
+            if deadline == cancel_scope._registered_deadline:
+                if cancel_scope in seen:
+                    continue
+                seen.add(cancel_scope)
+                pruned_heap.append((deadline, tiebreaker, cancel_scope))
+        # See test_cancel_scope_deadline_duplicates for a test that exercises
+        # this assert:
+        assert len(pruned_heap) == self._active
+        heapify(pruned_heap)
+        self._heap = pruned_heap
+
+    def expire(self, now):
+        did_something = False
+        while self._heap and self._heap[0][0] <= now:
+            deadline, _, cancel_scope = heappop(self._heap)
+            if deadline == cancel_scope._registered_deadline:
+                did_something = True
+                # This implicitly calls self.remove(), so we don't need to
+                # decrement _active here
+                cancel_scope.cancel()
+        # If we've accumulated too many stale entries, then prune the heap to
+        # keep it under control. (We only do this occasionally in a batch, to
+        # keep the amortized cost down)
+        if len(self._heap) > self._active * 2 + DEADLINE_HEAP_MIN_PRUNE_THRESHOLD:
+            self._prune()
+        return did_something
+
+
 @attr.s(eq=False, slots=True)
 class CancelStatus:
     """Tracks the cancellation status for a contiguous extent
@@ -518,13 +592,13 @@ class CancelScope(metaclass=Final):
                 self._registered_deadline = new
                 runner = GLOBAL_RUN_CONTEXT.runner
                 if runner.is_guest:
-                    old_next_deadline = runner.next_deadline()
+                    old_next_deadline = runner.deadlines.next_deadline()
                 if old != inf:
-                    del runner.deadlines[old, id(self)]
+                    runner.deadlines.remove(old, self)
                 if new != inf:
-                    runner.deadlines[new, id(self)] = self
+                    runner.deadlines.add(new, self)
                 if runner.is_guest:
-                    new_next_deadline = runner.next_deadline()
+                    new_next_deadline = runner.deadlines.next_deadline()
                     if old_next_deadline != new_next_deadline:
                         runner.force_guest_tick_asap()
 
@@ -583,7 +657,7 @@ class CancelScope(metaclass=Final):
         """
         return self._shield
 
-    @shield.setter
+    @shield.setter  # type: ignore  # "decorated property not supported"
     @enable_ki_protection
     def shield(self, new_value):
         if not isinstance(new_value, bool):
@@ -901,9 +975,6 @@ class Nursery(metaclass=NoPublicConstructor):
                   original function as the ``name=`` to make
                   debugging easier.
 
-        Returns:
-            True if successful, False otherwise.
-
         Raises:
             RuntimeError: If this nursery is no longer open
                           (i.e. its ``async with`` block has
@@ -912,7 +983,7 @@ class Nursery(metaclass=NoPublicConstructor):
         GLOBAL_RUN_CONTEXT.runner.spawn_impl(async_fn, args, self, name)
 
     async def start(self, async_fn, *args, name=None):
-        r""" Creates and initalizes a child task.
+        r"""Creates and initalizes a child task.
 
         Like :meth:`start_soon`, but blocks until the new task has
         finished initializing itself, and optionally returns some
@@ -924,7 +995,7 @@ class Nursery(metaclass=NoPublicConstructor):
 
         The conventional way to define ``async_fn`` is like::
 
-            async def async_fn(arg1, arg2, \*, task_status=trio.TASK_STATUS_IGNORED):
+            async def async_fn(arg1, arg2, *, task_status=trio.TASK_STATUS_IGNORED):
                 ...
                 task_status.started()
                 ...
@@ -1113,7 +1184,13 @@ class Task(metaclass=NoPublicConstructor):
 # The central Runner object
 ################################################################
 
-GLOBAL_RUN_CONTEXT = threading.local()
+
+class RunContext(threading.local):
+    runner: "Runner"
+    task: Task
+
+
+GLOBAL_RUN_CONTEXT = RunContext()
 
 
 @attr.s(frozen=True)
@@ -1151,7 +1228,8 @@ class GuestState:
     run_sync_soon_not_threadsafe = attr.ib()
     done_callback = attr.ib()
     unrolled_run_gen = attr.ib()
-    unrolled_run_next_send = attr.ib(factory=lambda: Value(None))
+    _value_factory: Callable[[], Value] = lambda: Value(None)
+    unrolled_run_next_send = attr.ib(factory=_value_factory, type=Outcome)
 
     def guest_tick(self):
         try:
@@ -1191,7 +1269,7 @@ class GuestState:
 @attr.s(eq=False, hash=False, slots=True)
 class Runner:
     clock = attr.ib()
-    instruments = attr.ib()
+    instruments: Instruments = attr.ib()
     io_manager = attr.ib()
     ki_manager = attr.ib()
 
@@ -1201,10 +1279,7 @@ class Runner:
     runq = attr.ib(factory=deque)
     tasks = attr.ib(factory=set)
 
-    # {(deadline, id(CancelScope)): CancelScope}
-    # only contains scopes with non-infinite deadlines that are currently
-    # attached to at least one task
-    deadlines = attr.ib(factory=SortedDict)
+    deadlines = attr.ib(factory=Deadlines)
 
     init_task = attr.ib(default=None)
     system_nursery = attr.ib(default=None)
@@ -1214,6 +1289,7 @@ class Runner:
 
     entry_queue = attr.ib(factory=EntryQueue)
     trio_token = attr.ib(default=None)
+    asyncgens = attr.ib(factory=AsyncGenerators)
 
     # If everything goes idle for this long, we call clock._autojump()
     clock_autojump_threshold = attr.ib(default=inf)
@@ -1231,19 +1307,12 @@ class Runner:
     def close(self):
         self.io_manager.close()
         self.entry_queue.close()
-        if self.instruments:
-            self.instrument("after_run")
+        self.asyncgens.close()
+        if "after_run" in self.instruments:
+            self.instruments.call("after_run")
         # This is where KI protection gets disabled, so we do it last
         self.ki_manager.close()
 
-    def next_deadline(self):
-        try:
-            (next_deadline, _), _ = self.deadlines.peekitem(0)
-        except IndexError:
-            return inf
-        else:
-            return next_deadline
-
     @_public
     def current_statistics(self):
         """Returns an object containing run-loop-level debugging information.
@@ -1268,7 +1337,7 @@ class Runner:
           other attributes vary between backends.
 
         """
-        seconds_to_next_deadline = self.next_deadline() - self.current_time()
+        seconds_to_next_deadline = self.deadlines.next_deadline() - self.current_time()
         return _RunStatistics(
             tasks_living=len(self.tasks),
             tasks_runnable=len(self.runq),
@@ -1292,9 +1361,7 @@ class Runner:
 
     @_public
     def current_clock(self):
-        """Returns the current :class:`~trio.abc.Clock`.
-
-        """
+        """Returns the current :class:`~trio.abc.Clock`."""
         return self.clock
 
     @_public
@@ -1341,8 +1408,8 @@ class Runner:
         if not self.runq and self.is_guest:
             self.force_guest_tick_asap()
         self.runq.append(task)
-        if self.instruments:
-            self.instrument("task_scheduled", task)
+        if "task_scheduled" in self.instruments:
+            self.instruments.call("task_scheduled", task)
 
     def spawn_impl(self, async_fn, args, nursery, name, *, system_task=False):
 
@@ -1391,7 +1458,7 @@ class Runner:
         # Set up the Task object
         ######
         task = Task._create(
-            coro=coro, parent_nursery=nursery, runner=self, name=name, context=context,
+            coro=coro, parent_nursery=nursery, runner=self, name=name, context=context
         )
 
         self.tasks.add(task)
@@ -1399,8 +1466,8 @@ class Runner:
             nursery._children.add(task)
             task._activate_cancel_status(nursery._cancel_status)
 
-        if self.instruments:
-            self.instrument("task_spawned", task)
+        if "task_spawned" in self.instruments:
+            self.instruments.call("task_spawned", task)
         # Special case: normally next_send should be an Outcome, but for the
         # very first send we have to send a literal unboxed None.
         self.reschedule(task, None)
@@ -1432,11 +1499,7 @@ class Runner:
 
         task._activate_cancel_status(None)
         self.tasks.remove(task)
-        if task is self.main_task:
-            self.main_task_outcome = outcome
-            self.system_nursery.cancel_scope.cancel()
-            self.system_nursery._child_finished(task, Value(None))
-        elif task is self.init_task:
+        if task is self.init_task:
             # If the init task crashed, then something is very wrong and we
             # let the error propagate. (It'll eventually be wrapped in a
             # TrioInternalError.)
@@ -1446,10 +1509,13 @@ class Runner:
             if self.tasks:  # pragma: no cover
                 raise TrioInternalError
         else:
+            if task is self.main_task:
+                self.main_task_outcome = outcome
+                outcome = Value(None)
             task._parent_nursery._child_finished(task, outcome)
 
-        if self.instruments:
-            self.instrument("task_exited", task)
+        if "task_exited" in self.instruments:
+            self.instruments.call("task_exited", task)
 
     ################
     # System tasks and init
@@ -1500,14 +1566,37 @@ class Runner:
         )
 
     async def init(self, async_fn, args):
-        async with open_nursery() as system_nursery:
-            self.system_nursery = system_nursery
-            try:
-                self.main_task = self.spawn_impl(async_fn, args, system_nursery, None)
-            except BaseException as exc:
-                self.main_task_outcome = Error(exc)
-                system_nursery.cancel_scope.cancel()
-            self.entry_queue.spawn()
+        # run_sync_soon task runs here:
+        async with open_nursery() as run_sync_soon_nursery:
+            # All other system tasks run here:
+            async with open_nursery() as self.system_nursery:
+                # Only the main task runs here:
+                async with open_nursery() as main_task_nursery:
+                    try:
+                        self.main_task = self.spawn_impl(
+                            async_fn, args, main_task_nursery, None
+                        )
+                    except BaseException as exc:
+                        self.main_task_outcome = Error(exc)
+                        return
+                    self.spawn_impl(
+                        self.entry_queue.task,
+                        (),
+                        run_sync_soon_nursery,
+                        "<TrioToken.run_sync_soon task>",
+                        system_task=True,
+                    )
+
+                # Main task is done; start shutting down system tasks
+                self.system_nursery.cancel_scope.cancel()
+
+            # System nursery is closed; finalize remaining async generators
+            await self.asyncgens.finalize_remaining(self)
+
+            # There are no more asyncgens, which means no more user-provided
+            # code except possibly run_sync_soon callbacks. It's finally safe
+            # to stop the run_sync_soon task and exit run().
+            run_sync_soon_nursery.cancel_scope.cancel()
 
     ################
     # Outside context problems
@@ -1641,64 +1730,6 @@ class Runner:
 
         await wait_task_rescheduled(abort)
 
-    ################
-    # Instrumentation
-    ################
-
-    def instrument(self, method_name, *args):
-        if not self.instruments:
-            return
-
-        for instrument in list(self.instruments):
-            try:
-                method = getattr(instrument, method_name)
-            except AttributeError:
-                continue
-            try:
-                method(*args)
-            except:
-                self.instruments.remove(instrument)
-                INSTRUMENT_LOGGER.exception(
-                    "Exception raised when calling %r on instrument %r. "
-                    "Instrument has been disabled.",
-                    method_name,
-                    instrument,
-                )
-
-    @_public
-    def add_instrument(self, instrument):
-        """Start instrumenting the current run loop with the given instrument.
-
-        Args:
-          instrument (trio.abc.Instrument): The instrument to activate.
-
-        If ``instrument`` is already active, does nothing.
-
-        """
-        if instrument not in self.instruments:
-            self.instruments.append(instrument)
-
-    @_public
-    def remove_instrument(self, instrument):
-        """Stop instrumenting the current run loop with the given instrument.
-
-        Args:
-          instrument (trio.abc.Instrument): The instrument to de-activate.
-
-        Raises:
-          KeyError: if the instrument is not currently active. This could
-              occur either because you never added it, or because you added it
-              and then it raised an unhandled exception and was automatically
-              deactivated.
-
-        """
-        # We're moving 'instruments' to being a set, so raise KeyError like
-        # set.remove does.
-        try:
-            self.instruments.remove(instrument)
-        except ValueError as exc:
-            raise KeyError(*exc.args)
-
 
 ################################################################
 # run
@@ -1777,7 +1808,7 @@ def setup_runner(clock, instruments, restrict_keyboard_interrupt_to_checkpoints)
 
     if clock is None:
         clock = SystemClock()
-    instruments = list(instruments)
+    instruments = Instruments(instruments)
     io_manager = TheIOManager()
     system_context = copy_context()
     system_context.run(current_async_library_cvar.set, "trio")
@@ -1790,6 +1821,7 @@ def setup_runner(clock, instruments, restrict_keyboard_interrupt_to_checkpoints)
         system_context=system_context,
         ki_manager=ki_manager,
     )
+    runner.asyncgens.install_hooks(runner)
 
     # This is where KI protection gets enabled, so we want to do it early - in
     # particular before we start modifying global state like GLOBAL_RUN_CONTEXT
@@ -1997,11 +2029,11 @@ def unrolled_run(runner, async_fn, args, host_uses_signal_set_wakeup_fd=False):
         if not host_uses_signal_set_wakeup_fd:
             runner.entry_queue.wakeup.wakeup_on_signals()
 
-        if runner.instruments:
-            runner.instrument("before_run")
+        if "before_run" in runner.instruments:
+            runner.instruments.call("before_run")
         runner.clock.start_clock()
         runner.init_task = runner.spawn_impl(
-            runner.init, (async_fn, args), None, "<init>", system_task=True,
+            runner.init, (async_fn, args), None, "<init>", system_task=True
         )
 
         # You know how people talk about "event loops"? This 'while' loop right
@@ -2009,11 +2041,9 @@ def unrolled_run(runner, async_fn, args, host_uses_signal_set_wakeup_fd=False):
         while runner.tasks:
             if runner.runq:
                 timeout = 0
-            elif runner.deadlines:
-                deadline, _ = runner.deadlines.keys()[0]
-                timeout = runner.clock.deadline_to_sleep_time(deadline)
             else:
-                timeout = _MAX_TIMEOUT
+                deadline = runner.deadlines.next_deadline()
+                timeout = runner.clock.deadline_to_sleep_time(deadline)
             timeout = min(max(0, timeout), _MAX_TIMEOUT)
 
             idle_primed = None
@@ -2029,27 +2059,21 @@ def unrolled_run(runner, async_fn, args, host_uses_signal_set_wakeup_fd=False):
                 timeout = runner.clock_autojump_threshold
                 idle_primed = IdlePrimedTypes.AUTOJUMP_CLOCK
 
-            if runner.instruments:
-                runner.instrument("before_io_wait", timeout)
+            if "before_io_wait" in runner.instruments:
+                runner.instruments.call("before_io_wait", timeout)
 
             # Driver will call io_manager.get_events(timeout) and pass it back
             # in throuh the yield
             events = yield timeout
             runner.io_manager.process_events(events)
 
-            if runner.instruments:
-                runner.instrument("after_io_wait", timeout)
+            if "after_io_wait" in runner.instruments:
+                runner.instruments.call("after_io_wait", timeout)
 
             # Process cancellations due to deadline expiry
             now = runner.clock.current_time()
-            while runner.deadlines:
-                (deadline, _), cancel_scope = runner.deadlines.peekitem(0)
-                if deadline <= now:
-                    # This removes the given scope from runner.deadlines:
-                    cancel_scope.cancel()
-                    idle_primed = None
-                else:
-                    break
+            if runner.deadlines.expire(now):
+                idle_primed = None
 
             # idle_primed != None means: if the IO wait hit the timeout, and
             # still nothing is happening, then we should start waking up
@@ -2121,8 +2145,8 @@ def unrolled_run(runner, async_fn, args, host_uses_signal_set_wakeup_fd=False):
                 task = batch.pop()
                 GLOBAL_RUN_CONTEXT.task = task
 
-                if runner.instruments:
-                    runner.instrument("before_task_step", task)
+                if "before_task_step" in runner.instruments:
+                    runner.instruments.call("before_task_step", task)
 
                 next_send_fn = task._next_send_fn
                 next_send = task._next_send
@@ -2184,8 +2208,8 @@ def unrolled_run(runner, async_fn, args, host_uses_signal_set_wakeup_fd=False):
                         runner.reschedule(task, exc)
                         task._next_send_fn = task.coro.throw
 
-                if runner.instruments:
-                    runner.instrument("after_task_step", task)
+                if "after_task_step" in runner.instruments:
+                    runner.instruments.call("after_task_step", task)
                 del GLOBAL_RUN_CONTEXT.task
 
     except GeneratorExit:
@@ -2284,8 +2308,17 @@ async def checkpoint():
     :func:`checkpoint`.)
 
     """
-    with CancelScope(deadline=-inf):
-        await _core.wait_task_rescheduled(lambda _: _core.Abort.SUCCEEDED)
+    # The scheduler is what checks timeouts and converts them into
+    # cancellations. So by doing the schedule point first, we ensure that the
+    # cancel point has the most up-to-date info.
+    await cancel_shielded_checkpoint()
+    task = current_task()
+    task._cancel_points += 1
+    if task._cancel_status.effectively_cancelled or (
+        task is task._runner.main_task and task._runner.ki_pending
+    ):
+        with CancelScope(deadline=-inf):
+            await _core.wait_task_rescheduled(lambda _: _core.Abort.SUCCEEDED)
 
 
 async def checkpoint_if_cancelled():
@@ -2312,16 +2345,17 @@ async def checkpoint_if_cancelled():
     task._cancel_points += 1
 
 
-if os.name == "nt":
+if sys.platform == "win32":
     from ._io_windows import WindowsIOManager as TheIOManager
     from ._generated_io_windows import *
-elif hasattr(select, "epoll"):
+elif sys.platform == "linux" or (not TYPE_CHECKING and hasattr(select, "epoll")):
     from ._io_epoll import EpollIOManager as TheIOManager
     from ._generated_io_epoll import *
-elif hasattr(select, "kqueue"):
+elif TYPE_CHECKING or hasattr(select, "kqueue"):
     from ._io_kqueue import KqueueIOManager as TheIOManager
     from ._generated_io_kqueue import *
 else:  # pragma: no cover
     raise NotImplementedError("unsupported platform")
 
 from ._generated_run import *
+from ._generated_instrumentation import *
