@@ -20,9 +20,11 @@ from .tutil import (
     check_sequence_matches,
     gc_collect_harder,
     ignore_coroutine_never_awaited_warnings,
+    buggy_pypy_asyncgens,
 )
 
 from ... import _core
+from .._run import DEADLINE_HEAP_MIN_PRUNE_THRESHOLD
 from ..._threads import to_thread_run_sync
 from ..._timeouts import sleep, fail_after
 from ...testing import (
@@ -130,7 +132,7 @@ async def test_basic_interleave():
         nursery.start_soon(looper, "b", record)
 
     check_sequence_matches(
-        record, [{("a", 0), ("b", 0)}, {("a", 1), ("b", 1)}, {("a", 2), ("b", 2)}],
+        record, [{("a", 0), ("b", 0)}, {("a", 1), ("b", 1)}, {("a", 2), ("b", 2)}]
     )
 
 
@@ -334,202 +336,6 @@ async def test_current_statistics(mock_clock):
     assert stats.seconds_to_next_deadline == inf
 
 
-@attr.s(eq=False, hash=False)
-class TaskRecorder:
-    record = attr.ib(factory=list)
-
-    def before_run(self):
-        self.record.append(("before_run",))
-
-    def task_scheduled(self, task):
-        self.record.append(("schedule", task))
-
-    def before_task_step(self, task):
-        assert task is _core.current_task()
-        self.record.append(("before", task))
-
-    def after_task_step(self, task):
-        assert task is _core.current_task()
-        self.record.append(("after", task))
-
-    def after_run(self):
-        self.record.append(("after_run",))
-
-    def filter_tasks(self, tasks):
-        for item in self.record:
-            if item[0] in ("schedule", "before", "after") and item[1] in tasks:
-                yield item
-            if item[0] in ("before_run", "after_run"):
-                yield item
-
-
-def test_instruments(recwarn):
-    r1 = TaskRecorder()
-    r2 = TaskRecorder()
-    r3 = TaskRecorder()
-
-    task = None
-
-    # We use a child task for this, because the main task does some extra
-    # bookkeeping stuff that can leak into the instrument results, and we
-    # don't want to deal with it.
-    async def task_fn():
-        nonlocal task
-        task = _core.current_task()
-
-        for _ in range(4):
-            await _core.checkpoint()
-        # replace r2 with r3, to test that we can manipulate them as we go
-        _core.remove_instrument(r2)
-        with pytest.raises(KeyError):
-            _core.remove_instrument(r2)
-        # add is idempotent
-        _core.add_instrument(r3)
-        _core.add_instrument(r3)
-        for _ in range(1):
-            await _core.checkpoint()
-
-    async def main():
-        async with _core.open_nursery() as nursery:
-            nursery.start_soon(task_fn)
-
-    _core.run(main, instruments=[r1, r2])
-
-    # It sleeps 5 times, so it runs 6 times.  Note that checkpoint()
-    # reschedules the task immediately upon yielding, before the
-    # after_task_step event fires.
-    expected = (
-        [("before_run",), ("schedule", task)]
-        + [("before", task), ("schedule", task), ("after", task)] * 5
-        + [("before", task), ("after", task), ("after_run",)]
-    )
-    assert len(r1.record) > len(r2.record) > len(r3.record)
-    assert r1.record == r2.record + r3.record
-    assert list(r1.filter_tasks([task])) == expected
-
-
-def test_instruments_interleave():
-    tasks = {}
-
-    async def two_step1():
-        tasks["t1"] = _core.current_task()
-        await _core.checkpoint()
-
-    async def two_step2():
-        tasks["t2"] = _core.current_task()
-        await _core.checkpoint()
-
-    async def main():
-        async with _core.open_nursery() as nursery:
-            nursery.start_soon(two_step1)
-            nursery.start_soon(two_step2)
-
-    r = TaskRecorder()
-    _core.run(main, instruments=[r])
-
-    expected = [
-        ("before_run",),
-        ("schedule", tasks["t1"]),
-        ("schedule", tasks["t2"]),
-        {
-            ("before", tasks["t1"]),
-            ("schedule", tasks["t1"]),
-            ("after", tasks["t1"]),
-            ("before", tasks["t2"]),
-            ("schedule", tasks["t2"]),
-            ("after", tasks["t2"]),
-        },
-        {
-            ("before", tasks["t1"]),
-            ("after", tasks["t1"]),
-            ("before", tasks["t2"]),
-            ("after", tasks["t2"]),
-        },
-        ("after_run",),
-    ]
-    print(list(r.filter_tasks(tasks.values())))
-    check_sequence_matches(list(r.filter_tasks(tasks.values())), expected)
-
-
-def test_null_instrument():
-    # undefined instrument methods are skipped
-    class NullInstrument:
-        pass
-
-    async def main():
-        await _core.checkpoint()
-
-    _core.run(main, instruments=[NullInstrument()])
-
-
-def test_instrument_before_after_run():
-    record = []
-
-    class BeforeAfterRun:
-        def before_run(self):
-            record.append("before_run")
-
-        def after_run(self):
-            record.append("after_run")
-
-    async def main():
-        pass
-
-    _core.run(main, instruments=[BeforeAfterRun()])
-    assert record == ["before_run", "after_run"]
-
-
-def test_instrument_task_spawn_exit():
-    record = []
-
-    class SpawnExitRecorder:
-        def task_spawned(self, task):
-            record.append(("spawned", task))
-
-        def task_exited(self, task):
-            record.append(("exited", task))
-
-    async def main():
-        return _core.current_task()
-
-    main_task = _core.run(main, instruments=[SpawnExitRecorder()])
-    assert ("spawned", main_task) in record
-    assert ("exited", main_task) in record
-
-
-# This test also tests having a crash before the initial task is even spawned,
-# which is very difficult to handle.
-def test_instruments_crash(caplog):
-    record = []
-
-    class BrokenInstrument:
-        def task_scheduled(self, task):
-            record.append("scheduled")
-            raise ValueError("oops")
-
-        def close(self):
-            # Shouldn't be called -- tests that the instrument disabling logic
-            # works right.
-            record.append("closed")  # pragma: no cover
-
-    async def main():
-        record.append("main ran")
-        return _core.current_task()
-
-    r = TaskRecorder()
-    main_task = _core.run(main, instruments=[r, BrokenInstrument()])
-    assert record == ["scheduled", "main ran"]
-    # the TaskRecorder kept going throughout, even though the BrokenInstrument
-    # was disabled
-    assert ("after", main_task) in r.record
-    assert ("after_run",) in r.record
-    # And we got a log message
-    exc_type, exc_value, exc_traceback = caplog.records[0].exc_info
-    assert exc_type is ValueError
-    assert str(exc_value) == "oops"
-    assert "Instrument has been disabled" in caplog.records[0].message
-
-
 async def test_cancel_scope_repr(mock_clock):
     scope = _core.CancelScope()
     assert "unbound" in repr(scope)
@@ -1526,6 +1332,33 @@ async def test_TrioToken_run_sync_soon_massive_queue():
     assert counter[0] == COUNT
 
 
+@pytest.mark.skipif(buggy_pypy_asyncgens, reason="PyPy 7.2 is buggy")
+def test_TrioToken_run_sync_soon_late_crash():
+    # Crash after system nursery is closed -- easiest way to do that is
+    # from an async generator finalizer.
+    record = []
+    saved = []
+
+    async def agen():
+        token = _core.current_trio_token()
+        try:
+            yield 1
+        finally:
+            token.run_sync_soon(lambda: {}["nope"])
+            token.run_sync_soon(lambda: record.append("2nd ran"))
+
+    async def main():
+        saved.append(agen())
+        await saved[-1].asend(None)
+        record.append("main exiting")
+
+    with pytest.raises(_core.TrioInternalError) as excinfo:
+        _core.run(main)
+
+    assert type(excinfo.value.__cause__) is KeyError
+    assert record == ["main exiting", "2nd ran"]
+
+
 async def test_slow_abort_basic():
     with _core.CancelScope() as scope:
         scope.cancel()
@@ -1729,7 +1562,7 @@ def test_nice_error_on_bad_calls_to_run_or_spawn():
             yield arg
 
         with pytest.raises(
-            TypeError, match="expected an async function but got an async generator",
+            TypeError, match="expected an async function but got an async generator"
         ):
             bad_call(async_gen, 0)
 
@@ -1861,7 +1694,7 @@ async def test_nursery_start(autojump_clock):
 
     # and if after the no-op started(), the child crashes, the error comes out
     # of start()
-    async def raise_keyerror_after_started(task_status=_core.TASK_STATUS_IGNORED,):
+    async def raise_keyerror_after_started(task_status=_core.TASK_STATUS_IGNORED):
         task_status.started()
         raise KeyError("whoopsiedaisy")
 
@@ -2013,7 +1846,7 @@ async def test_nursery_stop_async_iteration():
 
         async def __anext__(self):
             nexts = self.nexts
-            items = [None,] * len(nexts)
+            items = [None] * len(nexts)
             got_stop = False
 
             def handle(exc):
@@ -2351,3 +2184,14 @@ async def test_very_deep_cancel_scope_nesting():
         for _ in range(5000):
             exit_stack.enter_context(_core.CancelScope())
         outermost_scope.cancel()
+
+
+async def test_cancel_scope_deadline_duplicates():
+    # This exercises an assert in Deadlines._prune, by intentionally creating
+    # duplicate entries in the deadline heap.
+    now = _core.current_time()
+    with _core.CancelScope() as cscope:
+        for _ in range(DEADLINE_HEAP_MIN_PRUNE_THRESHOLD * 2):
+            cscope.deadline = now + 9998
+            cscope.deadline = now + 9999
+        await sleep(0.01)
