@@ -1,8 +1,9 @@
-import os as _os
-import sys as _sys
+import os
+import sys
 import select
 import socket as _stdlib_socket
 from functools import wraps as _wraps
+from typing import TYPE_CHECKING
 
 import idna as _idna
 
@@ -51,7 +52,7 @@ try:
 except ImportError:
     # As of at least 3.6, python on Windows is missing IPPROTO_IPV6
     # https://bugs.python.org/issue29515
-    if _sys.platform == "win32":
+    if sys.platform == "win32":  # pragma: no branch
         IPPROTO_IPV6 = 41
 
 ################################################################
@@ -239,14 +240,14 @@ def from_stdlib_socket(sock):
 
 @_wraps(_stdlib_socket.fromfd, assigned=(), updated=())
 def fromfd(fd, family, type, proto=0):
-    """Like :func:`socket.fromfd`, but returns a Trio socket object.
-
-    """
+    """Like :func:`socket.fromfd`, but returns a Trio socket object."""
     family, type, proto = _sniff_sockopts_for_fileno(family, type, proto, fd)
     return from_stdlib_socket(_stdlib_socket.fromfd(fd, family, type, proto))
 
 
-if hasattr(_stdlib_socket, "fromshare"):
+if sys.platform == "win32" or (
+    not TYPE_CHECKING and hasattr(_stdlib_socket, "fromshare")
+):
 
     @_wraps(_stdlib_socket.fromshare, assigned=(), updated=())
     def fromshare(*args, **kwargs):
@@ -287,13 +288,11 @@ def socket(
 
 
 def _sniff_sockopts_for_fileno(family, type, proto, fileno):
-    """Correct SOCKOPTS for given fileno, falling back to provided values.
-
-    """
+    """Correct SOCKOPTS for given fileno, falling back to provided values."""
     # Wrap the raw fileno into a Python socket object
     # This object might have the wrong metadata, but it lets us easily call getsockopt
     # and then we'll throw it away and construct a new one with the correct metadata.
-    if not _sys.platform == "linux":
+    if sys.platform != "linux":
         return family, type, proto
     from socket import SO_DOMAIN, SO_PROTOCOL, SOL_SOCKET, SO_TYPE
 
@@ -436,9 +435,7 @@ class _SocketType(SocketType):
         return repr(self._sock).replace("socket.socket", "trio.socket.socket")
 
     def dup(self):
-        """Same as :meth:`socket.socket.dup`.
-
-        """
+        """Same as :meth:`socket.socket.dup`."""
         return _SocketType(self._sock.dup())
 
     def close(self):
@@ -447,7 +444,7 @@ class _SocketType(SocketType):
             self._sock.close()
 
     async def bind(self, address):
-        address = await self._resolve_local_address(address)
+        address = await self._resolve_local_address_nocp(address)
         if (
             hasattr(_stdlib_socket, "AF_UNIX")
             and self.family == _stdlib_socket.AF_UNIX
@@ -461,6 +458,7 @@ class _SocketType(SocketType):
             # complete asynchronously, like connect. But in practice AFAICT
             # there aren't yet any real systems that do this, so we'll worry
             # about it when it happens.
+            await trio.lowlevel.checkpoint()
             return self._sock.bind(address)
 
     def shutdown(self, flag):
@@ -472,7 +470,7 @@ class _SocketType(SocketType):
 
     def is_readable(self):
         # use select.select on Windows, and select.poll everywhere else
-        if _sys.platform == "win32":
+        if sys.platform == "win32":
             rready, _, _ = select.select([self._sock], [], [], 0)
             return bool(rready)
         p = select.poll()
@@ -489,7 +487,9 @@ class _SocketType(SocketType):
     # Take an address in Python's representation, and returns a new address in
     # the same representation, but with names resolved to numbers,
     # etc.
-    async def _resolve_address(self, address, flags):
+    #
+    # NOTE: this function does not always checkpoint
+    async def _resolve_address_nocp(self, address, flags):
         # Do some pre-checking (or exit early for non-IP sockets)
         if self._sock.family == _stdlib_socket.AF_INET:
             if not isinstance(address, tuple) or not len(address) == 2:
@@ -500,14 +500,23 @@ class _SocketType(SocketType):
                     "address should be a (host, port, [flowinfo, [scopeid]]) tuple"
                 )
         elif self._sock.family == _stdlib_socket.AF_UNIX:
-            await trio.lowlevel.checkpoint()
             # unwrap path-likes
-            return _os.fspath(address)
-
+            return os.fspath(address)
         else:
-            await trio.lowlevel.checkpoint()
             return address
+
+        # -- From here on we know we have IPv4 or IPV6 --
         host, port, *_ = address
+        # Fast path for the simple case: already-resolved IP address,
+        # already-resolved port. This is particularly important for UDP, since
+        # every sendto call goes through here.
+        if isinstance(port, int):
+            try:
+                _stdlib_socket.inet_pton(self._sock.family, address[0])
+            except (OSError, TypeError):
+                pass
+            else:
+                return address
         # Special cases to match the stdlib, see gh-277
         if host == "":
             host = None
@@ -544,12 +553,16 @@ class _SocketType(SocketType):
         return normed
 
     # Returns something appropriate to pass to bind()
-    async def _resolve_local_address(self, address):
-        return await self._resolve_address(address, _stdlib_socket.AI_PASSIVE)
+    #
+    # NOTE: this function does not always checkpoint
+    async def _resolve_local_address_nocp(self, address):
+        return await self._resolve_address_nocp(address, _stdlib_socket.AI_PASSIVE)
 
     # Returns something appropriate to pass to connect()/sendto()/sendmsg()
-    async def _resolve_remote_address(self, address):
-        return await self._resolve_address(address, 0)
+    #
+    # NOTE: this function does not always checkpoint
+    async def _resolve_remote_address_nocp(self, address):
+        return await self._resolve_address_nocp(address, 0)
 
     async def _nonblocking_helper(self, fn, args, kwargs, wait_fn):
         # We have to reconcile two conflicting goals:
@@ -590,9 +603,7 @@ class _SocketType(SocketType):
     _accept = _make_simple_sock_method_wrapper("accept", _core.wait_readable)
 
     async def accept(self):
-        """Like :meth:`socket.socket.accept`, but async.
-
-        """
+        """Like :meth:`socket.socket.accept`, but async."""
         sock, addr = await self._accept()
         return from_stdlib_socket(sock), addr
 
@@ -606,7 +617,7 @@ class _SocketType(SocketType):
         # notification. This means it isn't really cancellable... we close the
         # socket if cancelled, to avoid confusion.
         try:
-            address = await self._resolve_remote_address(address)
+            address = await self._resolve_remote_address_nocp(address)
             async with _try_sync():
                 # An interesting puzzle: can a non-blocking connect() return EINTR
                 # (= raise InterruptedError)? PEP 475 specifically left this as
@@ -668,7 +679,7 @@ class _SocketType(SocketType):
         # Okay, the connect finished, but it might have failed:
         err = self._sock.getsockopt(_stdlib_socket.SOL_SOCKET, _stdlib_socket.SO_ERROR)
         if err != 0:
-            raise OSError(err, "Error in connect: " + _os.strerror(err))
+            raise OSError(err, "Error in connect: " + os.strerror(err))
 
     ################################################################
     # recv
@@ -726,13 +737,11 @@ class _SocketType(SocketType):
 
     @_wraps(_stdlib_socket.socket.sendto, assigned=(), updated=())
     async def sendto(self, *args):
-        """Similar to :meth:`socket.socket.sendto`, but async.
-
-        """
+        """Similar to :meth:`socket.socket.sendto`, but async."""
         # args is: data[, flags], address)
         # and kwargs are not accepted
         args = list(args)
-        args[-1] = await self._resolve_remote_address(args[-1])
+        args[-1] = await self._resolve_remote_address_nocp(args[-1])
         return await self._nonblocking_helper(
             _stdlib_socket.socket.sendto, args, {}, _core.wait_writable
         )
@@ -741,7 +750,9 @@ class _SocketType(SocketType):
     # sendmsg
     ################################################################
 
-    if hasattr(_stdlib_socket.socket, "sendmsg"):
+    if sys.platform != "win32" or (
+        not TYPE_CHECKING and hasattr(_stdlib_socket.socket, "sendmsg")
+    ):
 
         @_wraps(_stdlib_socket.socket.sendmsg, assigned=(), updated=())
         async def sendmsg(self, *args):
@@ -755,7 +766,7 @@ class _SocketType(SocketType):
             # and kwargs are not accepted
             if len(args) == 4 and args[-1] is not None:
                 args = list(args)
-                args[-1] = await self._resolve_remote_address(args[-1])
+                args[-1] = await self._resolve_remote_address_nocp(args[-1])
             return await self._nonblocking_helper(
                 _stdlib_socket.socket.sendmsg, args, {}, _core.wait_writable
             )
