@@ -73,9 +73,7 @@ class _UnboundedByteQueue:
             return self._get_impl(max_bytes)
 
 
-class MemorySendStream(
-    SendStream, metaclass=_util.SubclassingDeprecatedIn_v0_15_0
-):
+class MemorySendStream(SendStream, metaclass=_util.SubclassingDeprecatedIn_v0_15_0):
     """An in-memory :class:`~trio.abc.SendStream`.
 
     Args:
@@ -95,11 +93,12 @@ class MemorySendStream(
        you can change them at any time.
 
     """
+
     def __init__(
         self,
         send_all_hook=None,
         wait_send_all_might_not_block_hook=None,
-        close_hook=None
+        close_hook=None,
     ):
         self._conflict_detector = _util.ConflictDetector(
             "another task is using this stream"
@@ -156,9 +155,7 @@ class MemorySendStream(
             self.close_hook()
 
     async def aclose(self):
-        """Same as :meth:`close`, but async.
-
-        """
+        """Same as :meth:`close`, but async."""
         self.close()
         await _core.checkpoint()
 
@@ -208,6 +205,7 @@ class MemoryReceiveStream(
        change them at any time.
 
     """
+
     def __init__(self, receive_some_hook=None, close_hook=None):
         self._conflict_detector = _util.ConflictDetector(
             "another task is using this stream"
@@ -251,28 +249,20 @@ class MemoryReceiveStream(
             self.close_hook()
 
     async def aclose(self):
-        """Same as :meth:`close`, but async.
-
-        """
+        """Same as :meth:`close`, but async."""
         self.close()
         await _core.checkpoint()
 
     def put_data(self, data):
-        """Appends the given data to the internal buffer.
-
-        """
+        """Appends the given data to the internal buffer."""
         self._incoming.put(data)
 
     def put_eof(self):
-        """Adds an end-of-file marker to the internal buffer.
-
-        """
+        """Adds an end-of-file marker to the internal buffer."""
         self._incoming.close()
 
 
-def memory_stream_pump(
-    memory_send_stream, memory_receive_stream, *, max_bytes=None
-):
+def memory_stream_pump(memory_send_stream, memory_receive_stream, *, max_bytes=None):
     """Take data out of the given :class:`MemorySendStream`'s internal buffer,
     and put it into the given :class:`MemoryReceiveStream`'s internal buffer.
 
